import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.lang.Class;
import java.util.Hashtable;
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class EJBTestServlet extends HttpServlet
{
    // EJB Context factory for Allaire JRun.
    private static final String CTX_FACTORY = "allaire.ejipt.ContextFactory";
    private static final String CONTENT_TYPE = "text/html";

    protected Context ctx = null;

    /**Initialize global variables*/
    public void init(ServletConfig config) throws ServletException
    {
        super.init(config);

        // Establish the environment used to create the initial context
        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, CTX_FACTORY);
        env.put(Context.PROVIDER_URL, "ejipt://localhost:2323");

        try
        {
            // Create a context with default credentials
	        ctx = new InitialContext(env);
        }
        catch (NamingException ex)
        {
            throw new ServletException("Naming Exception: creating"
                + " InitialContext - " + ex.getMessage());
        }
    }


    /**Process the HTTP Get request*/
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException
    {
        // Get all of the information from the request.
        Integer months = new Integer(request.getParameter("months"));
        Float principal = new Float(request.getParameter("principal"));
        String custName = request.getParameter("name");
        String phoneNumber = request.getParameter("phone");
        Float payment = null;
        Float interestRate = null;

        CalculateLoanHome clHome = null;
        CalculateLoan cl = null;
        QuoteHome quoteHome = null;
        Quote quote = null;
        try
        {
            // Lookup the home interface for the CaluclateLoan Bean
            Object obj = ctx.lookup("CalculateLoan");
            clHome = (CalculateLoanHome)
                    PortableRemoteObject.narrow(obj,
                                        Class.forName("CalculateLoanHome"));

            // Create a new remote interface
            cl = clHome.create();

            // Calculate the monthly payment and get the current interest rate.
            payment = cl.calcMonthlyPayment(months, principal);
            interestRate = cl.getCurrentInterestRate();

            // Lookup the home interface for the Quote Bean
            obj = ctx.lookup("Quote");
            quoteHome = (QuoteHome)
                    PortableRemoteObject.narrow(obj,
                                        Class.forName("QuoteHome"));

            // Insert a new Quote into the database with the id of '1'
            quote = quoteHome.create(new Integer(1));
            // Update the database entry with all of the necessary information.
            quote.setCustomerName(custName);
            quote.setPhoneNumber(phoneNumber);
            quote.setLoanAmount(principal);
            quote.setMonthlyPayment(payment);
            quote.setInterestRate(interestRate);
        }
        catch (ClassNotFoundException ex)
        {
            throw new ServletException(ex);
        }
        catch (NamingException ex)
        {
            throw new ServletException(ex);
        }
        catch (CreateException ex)
        {
            throw new ServletException(ex);
        }
        catch (RemoteException ex)
        {
            throw new ServletException(ex);
        }

        // Create a simple screen as output from the servlet.
        response.setContentType(CONTENT_TYPE);
        PrintWriter out = response.getWriter();
        out.println("<html>");
        out.println("<head><title>EJBTestServlet</title></head>");
        out.println("<body>");
        out.println("<p>");
        out.println("The current interest rate is: " + interestRate + "<br>");
        out.println("For the loan amount of: " + principal + "<br>");
        out.println("Your monthly payment will be: " + payment + "<br>");
        out.println("</p>");
        out.println("</body></html>");
    }


    /**Clean up resources*/
    public void destroy()
    {
        try
        {
            ctx.close();
        }
        catch (NamingException ex)
        {
        }
    }
}
